@@ -10,6 +10,36 @@ function(
   assignments=file.choose(new=FALSE)
 ){
 
+  statAB<-function(A,B)
+  {
+    return(mean(A,na.rm=TRUE)-mean(B,na.rm=TRUE))
+  }
+  
+  statBA<-function(A,B)
+  {
+    return(mean(B,na.rm=TRUE)-mean(A,na.rm=TRUE))
+  }
+  
+  statabsAB<-function(A,B)
+  {
+    return(abs(mean(A,na.rm=TRUE)-mean(B,na.rm=TRUE)))
+  }
+  
+  statPAPB<-function(A1,B1,A2,B2)
+  {
+    return(mean(c(mean(A1,na.rm=TRUE),mean(A2,na.rm=TRUE)),na.rm=TRUE)-mean(c(mean(B1,na.rm=TRUE),mean(B2,na.rm=TRUE)),na.rm=TRUE))
+  }
+  
+  statPBPA<-function(A1,B1,A2,B2)
+  {
+    return(mean(c(mean(B1,na.rm=TRUE),mean(B2,na.rm=TRUE)),na.rm=TRUE)-mean(c(mean(A1,na.rm=TRUE),mean(A2,na.rm=TRUE)),na.rm=TRUE))
+  }
+  
+  statabsPAPB<-function(A1,B1,A2,B2)
+  {
+    return(abs(mean(c(mean(A1,na.rm=TRUE),mean(A2,na.rm=TRUE)),na.rm=TRUE)-mean(c(mean(B1,na.rm=TRUE),mean(B2,na.rm=TRUE)),na.rm=TRUE)))
+  }
+  
   if(design=="CRD"){
     if(save=="yes"){
       file<-file.choose(new=FALSE)
@@ -33,36 +63,30 @@ function(
     }
     ascores<-read.table(file.a)
     bscores<-read.table(file.b)
+    unlink(file.a,recursive=FALSE)
+    unlink(file.b,recursive=FALSE)
     
     ascores<-as.matrix(ascores)
-    mean.a<-numeric(number)
-    for(it in 1:number){
-      mean.a[it]<-mean(ascores[it,])
-    }
     bscores<-as.matrix(bscores)
-    mean.b<-numeric(number)
-    for(it in 1:number){
-      mean.b[it]<-mean(bscores[it,])
-    }
     distribution<-numeric(number)
     
     if(statistic=="A-B"){
       for(it in 1:number){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(ascores[it,],bscores[it,])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:number){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(ascores[it,],bscores[it,])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(ascores[it,],bscores[it,])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:number){
@@ -75,14 +99,15 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
     }
-    unlink(file.a,recursive=FALSE)
-    unlink(file.b,recursive=FALSE)
     return(p.value)
   }
   
@@ -119,36 +144,30 @@ function(
     }
     ascores<-read.table(file.a)
     bscores<-read.table(file.b)
+    unlink(file.a,recursive=FALSE)
+    unlink(file.b,recursive=FALSE)
     
     ascores<-as.matrix(ascores)
-    mean.a<-numeric(number)
-    for(it in 1:number){
-      mean.a[it]<-mean(ascores[it,])
-    }
     bscores<-as.matrix(bscores)
-    mean.b<-numeric(number)
-    for(it in 1:number){
-      mean.b[it]<-mean(bscores[it,])
-    }
     distribution<-numeric(number)
     
     if(statistic=="A-B"){
       for(it in 1:number){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(ascores[it,],bscores[it,])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:number){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(ascores[it,],bscores[it,])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(ascores[it,],bscores[it,])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:number){
@@ -161,14 +180,16 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
     }
-    unlink(file.a,recursive=FALSE)
-    unlink(file.b,recursive=FALSE)
+    
     return(p.value)
   }
   
@@ -224,36 +245,30 @@ function(
     }
     ascores<-read.table(file.a)
     bscores<-read.table(file.b)
+    unlink(file.a,recursive=FALSE)
+    unlink(file.b,recursive=FALSE)
     
     ascores<-as.matrix(ascores)
-    mean.a<-numeric(number)
-    for(it in 1:number){
-      mean.a[it]<-mean(ascores[it,])
-    }
     bscores<-as.matrix(bscores)
-    mean.b<-numeric(number)
-    for(it in 1:number){
-      mean.b[it]<-mean(bscores[it,])
-    }
     distribution<-numeric(number)
     
     if(statistic=="A-B"){
       for(it in 1:number){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(ascores[it,],bscores[it,])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:number){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(ascores[it,],bscores[it,])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(ascores[it,],bscores[it,])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:number){
@@ -266,14 +281,16 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
     }
-    unlink(file.a,recursive=FALSE)
-    unlink(file.b,recursive=FALSE)
+    
     return(p.value)
   }
   
@@ -301,21 +318,21 @@ function(
     
     if(statistic=="A-B"){
       for(it in 1:number){
-        distribution[it]<-mean(scores.a[[it]])-mean(scores.b[[it]])
+        distribution[it]<-statAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:number){
-        distribution[it]<-mean(scores.b[[it]])-mean(scores.a[[it]])
+        distribution[it]<-statBA(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number){
-        distribution[it]<-abs(mean(scores.a[[it]])-mean(scores.b[[it]]))
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:number){
@@ -328,8 +345,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE,append=FALSE)
@@ -378,81 +398,43 @@ function(
     for(it in 1:number){
       scores.a[[it]]<-c(scores.a1[[it]],scores.a2[[it]])
     }
-    mean.a<-numeric(number)
-    for(it in 1:number){
-      mean.a[it]<-mean(scores.a[[it]])
-    }
-    mean.b<-numeric(number)
-    for(it in 1:number){
-      mean.b[it]<-mean(scores.b1[[it]])
-    }
-    pmean.a<-numeric(number)
-    for(it in 1:number){
-      pmean.a[it]<-(mean(scores.a1[[it]])+mean(scores.a2[[it]]))/2
-    }
-    mean.a1<-numeric(number)
-    for(it in 1:number){
-      mean.a1[it]<-mean(scores.a1[[it]])
-    }
-    mean.a2<-numeric(number)
-    for(it in 1:number){
-      mean.a2[it]<-mean(scores.a2[[it]])
-    }
     distribution<-numeric(number)
     
     if(statistic=="A-B"){
       for(it in 1:number){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(scores.a[[it]],scores.b1[[it]])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b1)
+      observed.statistic<-statAB(observed.a,observed.b1)
     }
     else if(statistic=="B-A"){
       for(it in 1:number){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(scores.a[[it]],scores.b1[[it]])
       }
-      observed.statistic<-mean(observed.b1)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b1)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b1[[it]])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b1))
+      observed.statistic<-statabsAB(observed.a,observed.b1)
     }
     else if(statistic=="PA-PB"){
       for(it in 1:number){
-        distribution[it]<-pmean.a[it]-mean.b[it]
+        distribution[it]<-statPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],NA)
       }
-      observed.statistic<-((mean(observed.a1)+mean(observed.a2))/2)-mean(observed.b1)
+      observed.statistic<-statPAPB(observed.a1,observed.b1,observed.a2,NA)
     }
     else if(statistic=="PB-PA"){
       for(it in 1:number){
-        distribution[it]<-mean.b[it]-pmean.a[it]
+        distribution[it]<-statPBPA(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],NA)
       }
-      observed.statistic<-mean(observed.b1)-((mean(observed.a1)+mean(observed.a2))/2)
+      observed.statistic<-statPBPA(observed.a1,observed.b1,observed.a2,NA)
     }
     else if(statistic=="|PA-PB|"){
       for(it in 1:number){
-        distribution[it]<-abs(pmean.a[it]-mean.b[it])
+        distribution[it]<-statabsPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],NA)
       }
-      observed.statistic<-abs(((mean(observed.a1)+mean(observed.a2))/2)-mean(observed.b1))
-    }
-    else if(statistic=="AA-BB"){
-      for(it in 1:number){
-        distribution[it]<-(mean.a1[it]+mean.a2[it])-(mean.b[it])
-      }
-      observed.statistic<-(mean(observed.a1)+mean(observed.a2))-(mean(observed.b1))
-    }
-    else if(statistic=="BB-AA"){
-      for(it in 1:number){
-        distribution[it]<-mean.b[it]-(mean.a1[it]+mean.a2[it])
-      }
-      observed.statistic<-(mean(observed.b1))-(mean(observed.a1)+mean(observed.a2))
-    }
-    else if(statistic=="|AA-BB|"){
-      for(it in 1:number){
-        distribution[it]<-abs((mean.a1[it]+mean.a2[it])-(mean.b[it]))
-      }
-      observed.statistic<-abs((mean(observed.a1)+mean(observed.a2))-(mean(observed.b1)))
+      observed.statistic<-statabsPAPB(observed.a1,observed.b1,observed.a2,NA)
     }
     else{
       for(it in 1:number){
@@ -471,8 +453,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE,append=FALSE)
@@ -539,93 +524,43 @@ function(
     for(it in 1:number){
       scores.b[[it]]<-c(scores.b1[[it]],scores.b2[[it]])
     }
-    mean.a<-numeric(number)
-    for(it in 1:number){
-      mean.a[it]<-mean(scores.a[[it]])
-    }
-    mean.b<-numeric(number)
-    for(it in 1:number){
-      mean.b[it]<-mean(scores.b[[it]])
-    }	
-    pmean.a<-numeric(number)
-    for(it in 1:number){
-      pmean.a[it]<-(mean(scores.a1[[it]])+mean(scores.a2[[it]]))/2
-    }
-    pmean.b<-numeric(number)
-    for(it in 1:number){
-      pmean.b[it]<-(mean(scores.b1[[it]])+mean(scores.b2[[it]]))/2
-    }
-    mean.a1<-numeric(number)
-    for(it in 1:number){
-      mean.a1[it]<-mean(scores.a1[[it]])
-    }
-    mean.a2<-numeric(number)
-    for(it in 1:number){
-      mean.a2[it]<-mean(scores.a2[[it]])
-    }
-    mean.b1<-numeric(number)
-    for(it in 1:number){
-      mean.b1[it]<-mean(scores.b1[[it]])
-    }
-    mean.b2<-numeric(number)
-    for(it in 1:number){
-      mean.b2[it]<-mean(scores.b2[[it]])
-    }
     distribution<-numeric(number)
     
     if(statistic=="A-B"){
       for(it in 1:number){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:number){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else if(statistic=="PA-PB"){
       for(it in 1:number){
-        distribution[it]<-pmean.a[it]-pmean.b[it]
+        distribution[it]<-statPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],scores.b2[[it]])
       }
-      observed.statistic<-((mean(observed.a1)+mean(observed.a2))/2)-((mean(observed.b1)+mean(observed.b2))/2)
+      observed.statistic<-statPAPB(observed.a1,observed.b1,observed.a2,observed.b2)
     }
     else if(statistic=="PB-PA"){
       for(it in 1:number){
-        distribution[it]<-pmean.b[it]-pmean.a[it]
+        distribution[it]<-statPBPA(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],scores.b2[[it]])
       }
-      observed.statistic<-((mean(observed.b1)+mean(observed.b2))/2)-((mean(observed.a1)+mean(observed.a2))/2)
+      observed.statistic<-statPBPA(observed.a1,observed.b1,observed.a2,observed.b2)
     }
     else if(statistic=="|PA-PB|"){
       for(it in 1:number){
-        distribution[it]<-abs(pmean.a[it]-pmean.b[it])
+        distribution[it]<-statabsPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],scores.b2[[it]])
       }
-      observed.statistic<-abs(((mean(observed.a1)+mean(observed.a2))/2)-((mean(observed.b1)+mean(observed.b2))/2))
-    }
-    else if(statistic=="AA-BB"){
-      for(it in 1:number){
-        distribution[it]<-(mean.a1[it]+mean.a2[it])-(mean.b1[it]+mean.b2[it])
-      }
-      observed.statistic<-(mean(observed.a1)+mean(observed.a2))-(mean(observed.b1)+mean(observed.b2))
-    }
-    else if(statistic=="BB-AA"){
-      for(it in 1:number){
-        distribution[it]<-(mean.b1[it]+mean.b2[it])-(mean.a1[it]+mean.a2[it])
-      }
-      observed.statistic<-(mean(observed.b1)+mean(observed.b2))-(mean(observed.a1)+mean(observed.a2))
-    }
-    else if(statistic=="|AA-BB|"){
-      for(it in 1:number){
-        distribution[it]<-abs((mean.a1[it]+mean.a2[it])-(mean.b1[it]+mean.b2[it]))
-      }
-      observed.statistic<-abs((mean(observed.a1)+mean(observed.a2))-(mean(observed.b1)+mean(observed.b2)))
+      observed.statistic<-statabsPAPB(observed.a1,observed.b1,observed.a2,observed.b2)
     }
     else{
       for(it in 1:number){
@@ -646,8 +581,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE,append=FALSE)
@@ -684,6 +622,8 @@ function(
       if(nrow(combstartpts)==(number-1))break
     }
     
+    unlink(fileCOMBSTARTPTS,recursive=FALSE)
+    
     observed.a<-list()
     for(it in 1:N){
       observed.a[[it]]<-data[,it*2][data[,(it*2)-1]=="A"]
@@ -696,17 +636,17 @@ function(
     
     if(statistic=="A-B"){
       for(it in 1:N){
-        differences[it]<-mean(observed.a[[it]])-mean(observed.b[[it]])
+        differences[it]<-statAB(observed.a[[it]],observed.b[[it]])
       }
     }
     else if(statistic=="B-A"){
       for(it in 1:N){
-        differences[it]<-mean(observed.b[[it]])-mean(observed.a[[it]])
+        differences[it]<-statBA(observed.a[[it]],observed.b[[it]])
       }
     }
     else if(statistic=="|A-B|"){
       for(it in 1:N){
-        differences[it]<-abs(mean(observed.b[[it]])-mean(observed.a[[it]]))
+        differences[it]<-statabsAB(observed.a[[it]],observed.b[[it]])
       }
     }
     else{
@@ -716,7 +656,7 @@ function(
         differences[it]<-eval(parse(text=statistic))
       }
     }
-    observed.statistic<-mean(differences)
+    observed.statistic<-mean(differences,na.rm=TRUE)
     
     scores.a<-list(observed.a)
     for(iter in 1:(number-1)){
@@ -740,7 +680,7 @@ function(
       for(iter in 1:number){
         differ<-numeric(N)
         for(it in 1:N){
-          differ[it]<-mean(scores.a[[iter]][[it]])-mean(scores.b[[iter]][[it]])
+          differ[it]<-statAB(scores.a[[iter]][[it]],scores.b[[iter]][[it]])
         }
         differs[[iter]]<-differ
       }
@@ -749,7 +689,7 @@ function(
       for(iter in 1:number){
         differ<-numeric(N)
         for(it in 1:N){
-          differ[it]<-mean(scores.b[[iter]][[it]])-mean(scores.a[[iter]][[it]])
+          differ[it]<-statBA(scores.a[[iter]][[it]],scores.b[[iter]][[it]])
         }
         differs[[iter]]<-differ
       }
@@ -758,7 +698,7 @@ function(
       for(iter in 1:number){
         differ<-numeric(N)
         for(it in 1:N){
-          differ[it]<-abs(mean(scores.a[[iter]][[it]])-mean(scores.b[[iter]][[it]]))
+          differ[it]<-statabsAB(scores.a[[iter]][[it]],scores.b[[iter]][[it]])
         }
         differs[[iter]]<-differ
       }
@@ -777,10 +717,14 @@ function(
     
     distribution<-numeric(number)
     for(it in 1:number){
-      distribution[it]<-mean(differs[[it]])
+      distribution[it]<-mean(differs[[it]],na.rm=TRUE)
     }
+    
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     if(save=="yes"){
       fileSAVE<-file.choose(new=FALSE)
@@ -788,7 +732,7 @@ function(
     if(save=="yes"|save=="check"){
       write.table(distribution,file=fileSAVE,col.names=FALSE,row.names=FALSE,append=FALSE)
     }
-    unlink(fileCOMBSTARTPTS,recursive=FALSE)
+    
     return(p.value)
   }
   
@@ -814,18 +758,18 @@ function(
     distribution<-numeric(number)
     if(statistic=="A-B"){
       for(it in 1:number)
-        distribution[it]<-mean(scores.a[[it]])-mean(scores.b[[it]])
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+        distribution[it]<-statAB(scores.a[[it]],scores.b[[it]])
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:number)
-        distribution[it]<-mean(scores.b[[it]])-mean(scores.a[[it]])
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+        distribution[it]<-statBA(scores.a[[it]],scores.b[[it]])
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:number)
-        distribution[it]<-abs(mean(scores.a[[it]])-mean(scores.b[[it]]))
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b[[it]])
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:number){
@@ -838,8 +782,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/number
     
     if(save=="yes")
