@@ -8,6 +8,36 @@ function(
   starts=file.choose(new=FALSE),
   assignments=file.choose(new=FALSE)
 ){
+  
+  statAB<-function(A,B)
+  {
+    return(mean(A,na.rm=TRUE)-mean(B,na.rm=TRUE))
+  }
+  
+  statBA<-function(A,B)
+  {
+    return(mean(B,na.rm=TRUE)-mean(A,na.rm=TRUE))
+  }
+  
+  statabsAB<-function(A,B)
+  {
+    return(abs(mean(A,na.rm=TRUE)-mean(B,na.rm=TRUE)))
+  }
+  
+  statPAPB<-function(A1,B1,A2,B2)
+  {
+    return(mean(c(mean(A1,na.rm=TRUE),mean(A2,na.rm=TRUE)),na.rm=TRUE)-mean(c(mean(B1,na.rm=TRUE),mean(B2,na.rm=TRUE)),na.rm=TRUE))
+  }
+  
+  statPBPA<-function(A1,B1,A2,B2)
+  {
+    return(mean(c(mean(B1,na.rm=TRUE),mean(B2,na.rm=TRUE)),na.rm=TRUE)-mean(c(mean(A1,na.rm=TRUE),mean(A2,na.rm=TRUE)),na.rm=TRUE))
+  }
+  
+  statabsPAPB<-function(A1,B1,A2,B2)
+  {
+    return(abs(mean(c(mean(A1,na.rm=TRUE),mean(A2,na.rm=TRUE)),na.rm=TRUE)-mean(c(mean(B1,na.rm=TRUE),mean(B2,na.rm=TRUE)),na.rm=TRUE)))
+  }
 
   if(design=="CRD"){
     observed.a<-data[,2][data[,1]=="A"]
@@ -25,26 +55,26 @@ function(
       if(statistic %in% c("A-B","B-A","|A-B|")){
         mean.a<-numeric(ncol(scores.a))
         for(it in 1:ncol(scores.a)){
-          mean.a[it]<-mean(scores.a[,it])
+          mean.a[it]<-mean(scores.a[,it],na.rm=TRUE)
         }
         mean.b<-rev(mean.a)
         if(statistic=="A-B"){
           for(it in 1:length(mean.a)){
             distribution[it]<-mean.a[it]-mean.b[it]
           }
-          observed.statistic<-mean(observed.a)-mean(observed.b)
+          observed.statistic<-statAB(observed.a,observed.b)
         }
         else if(statistic=="B-A"){
           for(it in 1:length(mean.a)){
             distribution[it]<-mean.b[it]-mean.a[it]
           }
-          observed.statistic<-mean(observed.b)-mean(observed.a)
+          observed.statistic<-statBA(observed.a,observed.b)
         }
         else if(statistic=="|A-B|"){
           for(it in 1:length(mean.a)){
             distribution[it]<-abs(mean.a[it]-mean.b[it])
           }
-          observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+          observed.statistic<-statabsAB(observed.a,observed.b)
         }
       }
       else{
@@ -60,8 +90,11 @@ function(
         observed.statistic<-eval(parse(text=statistic))
       }
       
+      if(is.na(observed.statistic))
+        stop("Test statistic cannot be calculated. Please check the data.")
+      
       distribution<-sort(distribution)
-      test<-distribution>=observed.statistic
+      test<-is.na(distribution)|(distribution>=observed.statistic)
       p.value<-sum(test)/quantity
       if(save=="yes"|save=="check"){
         write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
@@ -121,21 +154,21 @@ function(
       
       if(statistic=="A-B"){
         for(it in 1:quantity){
-          distribution[it]<-mean(scores.a[it,])-mean(scores.b[it,])
+          distribution[it]<-statAB(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-mean(observed.a)-mean(observed.b)
+        observed.statistic<-statAB(observed.a,observed.b)
       }
       else if(statistic=="B-A"){
         for(it in 1:quantity){
-          distribution[it]<-mean(scores.b[it,])-mean(scores.a[it,])
+          distribution[it]<-statBA(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-mean(observed.b)-mean(observed.a)
+        observed.statistic<-statBA(observed.a,observed.b)
       }
       else if(statistic=="|A-B|"){
         for(it in 1:quantity){
-          distribution[it]<-abs(mean(scores.a[it,])-mean(scores.b[it,]))
+          distribution[it]<-statabsAB(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+        observed.statistic<-statabsAB(observed.a,observed.b)
       }
       else{
         for(it in 1:quantity){
@@ -148,12 +181,17 @@ function(
         observed.statistic<-eval(parse(text=statistic))
       }
       
-      distribution<-sort(distribution)
-      test<-distribution>=observed.statistic
-      p.value<-sum(test)/quantity
       unlink(fileCRD,recursive=FALSE)
       unlink(file.a,recursive=FALSE)
       unlink(file.b,recursive=FALSE)
+      
+      if(is.na(observed.statistic))
+        stop("Test statistic cannot be calculated. Please check the data.")
+      
+      distribution<-sort(distribution)
+      test<-is.na(distribution)|(distribution>=observed.statistic)
+      p.value<-sum(test)/quantity
+      
       if(save=="yes"|save=="check"){
         write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
       }
@@ -183,21 +221,21 @@ function(
     
     if(statistic=="A-B"){
       for(it in 1:quantity){
-        distribution[it]<-mean(scores.a[it,])-mean(scores.b[it,])
+        distribution[it]<-statAB(scores.a[it,],scores.b[it,])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:quantity){
-        distribution[it]<-mean(scores.b[it,])-mean(scores.a[it,])
+        distribution[it]<-statBA(scores.a[it,],scores.b[it,])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:quantity){
-        distribution[it]<-abs(mean(scores.a[it,])-mean(scores.b[it,]))
+        distribution[it]<-statabsAB(scores.a[it,],scores.b[it,])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:quantity){
@@ -210,8 +248,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/quantity
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
@@ -295,21 +336,21 @@ function(
       
       if(statistic=="A-B"){
         for(it in 1:quantity){
-          distribution[it]<-mean(scores.a[it,])-mean(scores.b[it,])
+          distribution[it]<-statAB(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-mean(observed.a)-mean(observed.b)
+        observed.statistic<-statAB(observed.a,observed.b)
       }
       else if(statistic=="B-A"){
         for(it in 1:quantity){
-          distribution[it]<-mean(scores.b[it,])-mean(scores.a[it,])
+          distribution[it]<-statBA(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-mean(observed.b)-mean(observed.a)
+        observed.statistic<-statBA(observed.a,observed.b)
       }
       else if(statistic=="|A-B|"){
         for(it in 1:quantity){
-          distribution[it]<-abs(mean(scores.a[it,])-mean(scores.b[it,]))
+          distribution[it]<-statabsAB(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+        observed.statistic<-statabsAB(observed.a,observed.b)
       }
       else{
         for(it in 1:quantity){
@@ -322,8 +363,11 @@ function(
         observed.statistic<-eval(parse(text=statistic))
       }
       
+      if(is.na(observed.statistic))
+        stop("Test statistic cannot be calculated. Please check the data.")
+      
       distribution<-sort(distribution)
-      test<-distribution>=observed.statistic
+      test<-is.na(distribution)|(distribution>=observed.statistic)
       p.value<-sum(test)/quantity
       if(save=="yes"|save=="check"){
         write.table(distribution,file=file,row.names=FALSE,col.names=FALSE)
@@ -403,6 +447,11 @@ function(
         }
         if(nrow(CRD)==quantityCRD)break
       }
+      
+      unlink(fileCRD,recursive=FALSE)
+      unlink(file.a,recursive=FALSE)
+      unlink(file.b,recursive=FALSE)
+      
       scores.a<-as.matrix(scores.a)
       scores.b<-as.matrix(scores.b)
       quantity<-nrow(scores.a)
@@ -410,21 +459,21 @@ function(
       
       if(statistic=="A-B"){
         for(it in 1:quantity){
-          distribution[it]<-mean(scores.a[it,])-mean(scores.b[it,])
+          distribution[it]<-statAB(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-mean(observed.a)-mean(observed.b)
+        observed.statistic<-statAB(observed.a,observed.b)
       }
       else if(statistic=="B-A"){
         for(it in 1:quantity){
-          distribution[it]<-mean(scores.b[it,])-mean(scores.a[it,])
+          distribution[it]<-statBA(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-mean(observed.b)-mean(observed.a)
+        observed.statistic<-statBA(observed.a,observed.b)
       }
       else if(statistic=="|A-B|"){
         for(it in 1:quantity){
-          distribution[it]<-abs(mean(scores.a[it,])-mean(scores.b[it,]))
+          distribution[it]<-statabsAB(scores.a[it,],scores.b[it,])
         }
-        observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+        observed.statistic<-statabsAB(observed.a,observed.b)
       }
       else{
         for(it in 1:quantity){
@@ -437,12 +486,12 @@ function(
         observed.statistic<-eval(parse(text=statistic))
       }
       
+      if(is.na(observed.statistic))
+        stop("Test statistic cannot be calculated. Please check the data.")
+      
       distribution<-sort(distribution)
-      test<-distribution>=observed.statistic
+      test<-is.na(distribution)|(distribution>=observed.statistic)
       p.value<-sum(test)/quantity
-      unlink(fileCRD,recursive=FALSE)
-      unlink(file.a,recursive=FALSE)
-      unlink(file.b,recursive=FALSE)
       if(save=="yes"|save=="check"){
         write.table(distribution,file=file,col.names=FALSE,row.names=FALSE)
       }
@@ -472,21 +521,21 @@ function(
     
     if(statistic=="A-B"){
       for(it in 1:quantity){
-        distribution[it]<-mean(scores.a[[it]])-mean(scores.b[[it]])
+        distribution[it]<-statAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:quantity){
-        distribution[it]<-mean(scores.b[[it]])-mean(scores.a[[it]])
+        distribution[it]<-statBA(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:quantity){
-        distribution[it]<-abs(mean(scores.a[[it]])-mean(scores.b[[it]]))
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:quantity){
@@ -499,8 +548,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/quantity
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE,append=FALSE)
@@ -547,81 +599,43 @@ function(
     for(it in 1:quantity){
       scores.a[[it]]<-c(scores.a1[[it]],scores.a2[[it]])
     }
-    mean.a<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.a[it]<-mean(scores.a[[it]])
-    }
-    mean.b<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.b[it]<-mean(scores.b1[[it]])
-    }
-    pmean.a<-numeric(quantity)
-    for(it in 1:quantity){
-      pmean.a[it]<-(mean(scores.a1[[it]])+mean(scores.a2[[it]]))/2
-    }
-    mean.a1<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.a1[it]<-mean(scores.a1[[it]])
-    }
-    mean.a2<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.a2[it]<-mean(scores.a2[[it]])
-    }
     distribution<-numeric(quantity)
     
     if(statistic=="A-B"){	
       for(it in 1:quantity){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(scores.a[[it]],scores.b1[[it]])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b1)
+      observed.statistic<-statAB(observed.a,observed.b1)
     }
     else if(statistic=="B-A"){
       for(it in 1:quantity){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(scores.a[[it]],scores.b1[[it]])
       }
-      observed.statistic<-mean(observed.b1)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b1)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:quantity){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b1[[it]])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b1))
+      observed.statistic<-statabsAB(observed.a,observed.b1)
     }
     else if(statistic=="PA-PB"){
       for(it in 1:quantity){
-        distribution[it]<-pmean.a[it]-mean.b[it]
+        distribution[it]<-statPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],NA)
       }
-      observed.statistic<-((mean(observed.a1)+mean(observed.a2))/2)-mean(observed.b1)
+      observed.statistic<-statPAPB(observed.a1,observed.b1,observed.a2,NA)
     }
     else if(statistic=="PB-PA"){
       for(it in 1:quantity){
-        distribution[it]<-mean.b[it]-pmean.a[it]
+        distribution[it]<-statPBPA(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],NA)
       }
-      observed.statistic<-mean(observed.b1)-((mean(observed.a1)+mean(observed.a2))/2)
+      observed.statistic<-statPBPA(observed.a1,observed.b1,observed.a2,NA)
     }
     else if(statistic=="|PA-PB|"){
       for(it in 1:quantity){
-        distribution[it]<-abs(pmean.a[it]-mean.b[it])
+        distribution[it]<-statabsPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],NA)
       }
-      observed.statistic<-abs(((mean(observed.a1)+mean(observed.a2))/2)-mean(observed.b1))
-    }
-    else if(statistic=="AA-BB"){
-      for(it in 1:quantity){
-        distribution[it]<-(mean.a1[it]+mean.a2[it])-(mean.b[it])
-      }
-      observed.statistic<-(mean(observed.a1)+mean(observed.a2))-(mean(observed.b1))
-    }
-    else if(statistic=="BB-AA"){
-      for(it in 1:quantity){
-        distribution[it]<-mean.b[it]-(mean.a1[it]+mean.a2[it])
-      }
-      observed.statistic<-(mean(observed.b1))-(mean(observed.a1)+mean(observed.a2))
-    }
-    else if(statistic=="|AA-BB|"){
-      for(it in 1:quantity){
-        distribution[it]<-abs((mean.a1[it]+mean.a2[it])-(mean.b[it]))
-      }
-      observed.statistic<-abs((mean(observed.a1)+mean(observed.a2))-(mean(observed.b1)))
+      observed.statistic<-statabsPAPB(observed.a1,observed.b1,observed.a2,NA)
     }
     else{
       for(it in 1:quantity){
@@ -640,8 +654,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/quantity
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE,append=FALSE)
@@ -706,93 +723,44 @@ function(
     for(it in 1:quantity){
       scores.b[[it]]<-c(scores.b1[[it]],scores.b2[[it]])
     }
-    mean.a<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.a[it]<-mean(scores.a[[it]])
-    }
-    mean.b<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.b[it]<-mean(scores.b[[it]])
-    }
-    pmean.a<-numeric(quantity)
-    for(it in 1:quantity){
-      pmean.a[it]<-(mean(scores.a1[[it]])+mean(scores.a2[[it]]))/2
-    }
-    pmean.b<-numeric(quantity)
-    for(it in 1:quantity){
-      pmean.b[it]<-(mean(scores.b1[[it]])+mean(scores.b2[[it]]))/2
-    }
-    mean.a1<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.a1[it]<-mean(scores.a1[[it]])
-    }
-    mean.a2<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.a2[it]<-mean(scores.a2[[it]])
-    }
-    mean.b1<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.b1[it]<-mean(scores.b1[[it]])
-    }
-    mean.b2<-numeric(quantity)
-    for(it in 1:quantity){
-      mean.b2[it]<-mean(scores.b2[[it]])
-    }
+
     distribution<-numeric(quantity)
     
     if(statistic=="A-B"){
       for(it in 1:quantity){
-        distribution[it]<-mean.a[it]-mean.b[it]
+        distribution[it]<-statAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:quantity){
-        distribution[it]<-mean.b[it]-mean.a[it]
+        distribution[it]<-statBA(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:quantity){
-        distribution[it]<-abs(mean.a[it]-mean.b[it])
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b[[it]])
       }
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else if(statistic=="PA-PB"){
       for(it in 1:quantity){
-        distribution[it]<-pmean.a[it]-pmean.b[it]
+        distribution[it]<-statPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],scores.b2[[it]])
       }
-      observed.statistic<-((mean(observed.a1)+mean(observed.a2))/2)-((mean(observed.b1)+mean(observed.b2))/2)
+      observed.statistic<-statPAPB(observed.a1,observed.b1,observed.a2,observed.b2)
     }
     else if(statistic=="PB-PA"){
       for(it in 1:quantity){
-        distribution[it]<-pmean.b[it]-pmean.a[it]
+        distribution[it]<-statPBPA(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],scores.b2[[it]])
       }
-      observed.statistic<-((mean(observed.b1)+mean(observed.b2))/2)-((mean(observed.a1)+mean(observed.a2))/2)
+      observed.statistic<-statPBPA(observed.a1,observed.b1,observed.a2,observed.b2)
     }
     else if(statistic=="|PA-PB|"){
       for(it in 1:quantity){
-        distribution[it]<-abs(pmean.a[it]-pmean.b[it])
+        distribution[it]<-statabsPAPB(scores.a1[[it]],scores.b1[[it]],scores.a2[[it]],scores.b2[[it]])
       }
-      observed.statistic<-abs(((mean(observed.a1)+mean(observed.a2))/2)-((mean(observed.b1)+mean(observed.b2))/2))
-    }
-    else if(statistic=="AA-BB"){
-      for(it in 1:quantity){
-        distribution[it]<-(mean.a1[it]+mean.a2[it])-(mean.b1[it]+mean.b2[it])
-      }
-      observed.statistic<-(mean(observed.a1)+mean(observed.a2))-(mean(observed.b1)+mean(observed.b2))
-    }
-    else if(statistic=="BB-AA"){
-      for(it in 1:quantity){
-        distribution[it]<-(mean.b1[it]+mean.b2[it])-(mean.a1[it]+mean.a2[it])
-      }
-      observed.statistic<-(mean(observed.b1)+mean(observed.b2))-(mean(observed.a1)+mean(observed.a2))
-    }
-    else if(statistic=="|AA-BB|"){
-      for(it in 1:quantity){
-        distribution[it]<-abs((mean.a1[it]+mean.a2[it])-(mean.b1[it]+mean.b2[it]))
-      }
-      observed.statistic<-abs((mean(observed.a1)+mean(observed.a2))-(mean(observed.b1)+mean(observed.b2)))
+      observed.statistic<-statabsPAPB(observed.a1,observed.b1,observed.a2,observed.b2)
     }
     else{
       for(it in 1:quantity){
@@ -813,8 +781,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/quantity
     if(save=="yes"|save=="check"){
       write.table(distribution,file=file,col.names=FALSE,row.names=FALSE,append=FALSE)
@@ -914,6 +885,10 @@ function(
     write.table(combstartpts,file=fileASSIGNMENTS,col.names=FALSE,row.names=FALSE)
     assignments<-read.table(fileASSIGNMENTS)
     
+    unlink(fileSTARTPTS,recursive=FALSE)
+    unlink(fileCOMBSTARTPOINTS,recursive=FALSE)
+    unlink(fileASSIGNMENTS,recursive=FALSE)
+    
     observed.a<-list()
     for(it in 1:N){
       observed.a[[it]]<-data[,it*2][data[,(it*2)-1]=="A"]
@@ -926,17 +901,17 @@ function(
     
     if(statistic=="A-B"){
       for(it in 1:N){
-        differences[it]<-mean(observed.a[[it]])-mean(observed.b[[it]])
+        differences[it]<-statAB(observed.a[[it]],observed.b[[it]])
       }
     }
     else if(statistic=="B-A"){
       for(it in 1:N){
-        differences[it]<-mean(observed.b[[it]])-mean(observed.a[[it]])
+        differences[it]<-statBA(observed.a[[it]],observed.b[[it]])
       }
     }
     else if(statistic=="|A-B|"){
       for(it in 1:N){
-        differences[it]<-abs(mean(observed.b[[it]])-mean(observed.a[[it]]))
+        differences[it]<-statabsAB(observed.a[[it]],observed.b[[it]])
       }
     }
     else{
@@ -947,7 +922,8 @@ function(
       }
     }
     
-    observed.statistic<-mean(differences)
+    observed.statistic<-mean(differences,na.rm=TRUE)
+    
     scores.a<-list()
     for(iter in 1:nrow(assignments)){
       ascores<-list()
@@ -970,7 +946,7 @@ function(
       for(iter in 1:nrow(assignments)){
         differ<-numeric(N)
         for(it in 1:N){
-          differ[it]<-mean(scores.a[[iter]][[it]])-mean(scores.b[[iter]][[it]])
+          differ[it]<-statAB(scores.a[[iter]][[it]],scores.b[[iter]][[it]])
         }
         differs[[iter]]<-differ
       }
@@ -979,7 +955,7 @@ function(
       for(iter in 1:nrow(assignments)){
         differ<-numeric(N)
         for(it in 1:N){
-          differ[it]<-mean(scores.b[[iter]][[it]])-mean(scores.a[[iter]][[it]])
+          differ[it]<-statBA(scores.a[[iter]][[it]],scores.b[[iter]][[it]])
         }
         differs[[iter]]<-differ
       }
@@ -988,7 +964,7 @@ function(
       for(iter in 1:nrow(assignments)){
         differ<-numeric(N)
         for(it in 1:N){
-          differ[it]<-abs(mean(scores.a[[iter]][[it]])-mean(scores.b[[iter]][[it]]))
+          differ[it]<-statabsAB(scores.a[[iter]][[it]],scores.b[[iter]][[it]])
         }
         differs[[iter]]<-differ
       }
@@ -1007,10 +983,14 @@ function(
     
     distribution<-numeric(nrow(assignments))
     for(it in 1:nrow(assignments)){
-      distribution[it]<-mean(differs[[it]])
+      distribution[it]<-mean(differs[[it]],na.rm=TRUE)
     }
+    
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/nrow(assignments)
     if(save=="yes"){
       fileSAVE<-file.choose(new=FALSE)
@@ -1018,9 +998,7 @@ function(
     if(save=="yes"|save=="check"){
       write.table(distribution,file=fileSAVE,col.names=FALSE,row.names=FALSE,append=FALSE)
     }
-    unlink(fileSTARTPTS,recursive=FALSE)
-    unlink(fileCOMBSTARTPOINTS,recursive=FALSE)
-    unlink(fileASSIGNMENTS,recursive=FALSE)
+    
     return(p.value)
   }
   
@@ -1046,18 +1024,18 @@ function(
     distribution<-numeric(quantity)
     if(statistic=="A-B"){
       for(it in 1:quantity)
-        distribution[it]<-mean(scores.a[[it]])-mean(scores.b[[it]])
-      observed.statistic<-mean(observed.a)-mean(observed.b)
+        distribution[it]<-statAB(scores.a[[it]],scores.b[[it]])
+      observed.statistic<-statAB(observed.a,observed.b)
     }
     else if(statistic=="B-A"){
       for(it in 1:quantity)
-        distribution[it]<-mean(scores.b[[it]])-mean(scores.a[[it]])
-      observed.statistic<-mean(observed.b)-mean(observed.a)
+        distribution[it]<-statBA(scores.a[[it]],scores.b[[it]])
+      observed.statistic<-statBA(observed.a,observed.b)
     }
     else if(statistic=="|A-B|"){
       for(it in 1:quantity)
-        distribution[it]<-abs(mean(scores.a[[it]])-mean(scores.b[[it]]))
-      observed.statistic<-abs(mean(observed.a)-mean(observed.b))
+        distribution[it]<-statabsAB(scores.a[[it]],scores.b[[it]])
+      observed.statistic<-statabsAB(observed.a,observed.b)
     }
     else{
       for(it in 1:quantity){
@@ -1070,8 +1048,11 @@ function(
       observed.statistic<-eval(parse(text=statistic))
     }
     
+    if(is.na(observed.statistic))
+      stop("Test statistic cannot be calculated. Please check the data.")
+    
     distribution<-sort(distribution)
-    test<-distribution>=observed.statistic
+    test<-is.na(distribution)|(distribution>=observed.statistic)
     p.value<-sum(test)/quantity
     
     if(save=="yes")
